library benchpress.src.metric.multi_metric;

import "package:angular2/src/core/di.dart"
    show bind, provide, Binding, Provider, Injector, OpaqueToken;
import "package:angular2/src/facade/collection.dart" show StringMapWrapper;
import "package:angular2/src/facade/async.dart" show Future, PromiseWrapper;
import "../metric.dart" show Metric;

class MultiMetric extends Metric {
  List<Metric> _metrics;
  static List<Provider> createBindings(List<dynamic> childTokens) {
    return [
      bind(_CHILDREN).toFactory(
          (Injector injector) =>
              childTokens.map((token) => injector.get(token)).toList(),
          [Injector]),
      bind(MultiMetric)
          .toFactory((children) => new MultiMetric(children), [_CHILDREN])
    ];
  }

  MultiMetric(this._metrics) : super() {
    /* super call moved to initializer */;
  }
  /**
   * Starts measuring
   */
  Future<dynamic> beginMeasure() {
    return PromiseWrapper
        .all(this._metrics.map((metric) => metric.beginMeasure()).toList());
  }

  /**
   * Ends measuring and reports the data
   * since the begin call.
   * @param restart: Whether to restart right after this.
   */
  Future<Map<String, dynamic>> endMeasure(bool restart) {
    return PromiseWrapper
        .all(this._metrics.map((metric) => metric.endMeasure(restart)).toList())
        .then((values) => mergeStringMaps(values));
  }

  /**
   * Describes the metrics provided by this metric implementation.
   * (e.g. units, ...)
   */
  Map<String, dynamic> describe() {
    return mergeStringMaps(
        this._metrics.map((metric) => metric.describe()).toList());
  }
}

Object mergeStringMaps(List<Map<String, String>> maps) {
  var result = {};
  maps.forEach((map) {
    StringMapWrapper.forEach(map, (value, prop) {
      result[prop] = value;
    });
  });
  return result;
}

var _CHILDREN = new OpaqueToken("MultiMetric.children");
