library benchpress.src.runner;

import "package:angular2/src/core/di.dart"
    show Injector, bind, provide, Provider;
import "package:angular2/src/facade/lang.dart" show isPresent, isBlank;
import "package:angular2/src/facade/async.dart" show Future, PromiseWrapper;
import "sampler.dart" show Sampler, SampleState;
import "reporter/console_reporter.dart" show ConsoleReporter;
import "reporter/multi_reporter.dart" show MultiReporter;
import "validator/regression_slope_validator.dart"
    show RegressionSlopeValidator;
import "validator/size_validator.dart" show SizeValidator;
import "validator.dart" show Validator;
import "metric/perflog_metric.dart" show PerflogMetric;
import "metric/multi_metric.dart" show MultiMetric;
import "webdriver/chrome_driver_extension.dart" show ChromeDriverExtension;
import "webdriver/firefox_driver_extension.dart" show FirefoxDriverExtension;
import "webdriver/ios_driver_extension.dart" show IOsDriverExtension;
import "web_driver_extension.dart" show WebDriverExtension;
import "sample_description.dart" show SampleDescription;
import "web_driver_adapter.dart" show WebDriverAdapter;
import "reporter.dart" show Reporter;
import "metric.dart" show Metric;
import "common_options.dart" show Options;

/**
 * The Runner is the main entry point for executing a sample run.
 * It provides defaults, creates the injector and calls the sampler.
 */
class Runner {
  List<Provider> _defaultBindings;
  Runner([List<Provider> defaultBindings = null]) {
    if (isBlank(defaultBindings)) {
      defaultBindings = [];
    }
    this._defaultBindings = defaultBindings;
  }
  Future<SampleState> sample({id, execute, prepare, microMetrics, bindings}) {
    var sampleBindings = [
      _DEFAULT_PROVIDERS,
      this._defaultBindings,
      bind(Options.SAMPLE_ID).toValue(id),
      bind(Options.EXECUTE).toValue(execute)
    ];
    if (isPresent(prepare)) {
      sampleBindings.add(bind(Options.PREPARE).toValue(prepare));
    }
    if (isPresent(microMetrics)) {
      sampleBindings.add(bind(Options.MICRO_METRICS).toValue(microMetrics));
    }
    if (isPresent(bindings)) {
      sampleBindings.add(bindings);
    }
    var inj = Injector.resolveAndCreate(sampleBindings);
    var adapter = inj.get(WebDriverAdapter);
    return PromiseWrapper.all([
      adapter.capabilities(),
      adapter.executeScript("return window.navigator.userAgent;")
    ]).then((args) {
      var capabilities = args[0];
      var userAgent = args[1];
      // This might still create instances twice. We are creating a new injector with all the

      // providers.

      // Only WebDriverAdapter is reused.

      // TODO vsavkin consider changing it when toAsyncFactory is added back or when child

      // injectors are handled better.
      var injector = Injector.resolveAndCreate([
        sampleBindings,
        bind(Options.CAPABILITIES).toValue(capabilities),
        bind(Options.USER_AGENT).toValue(userAgent),
        provide(WebDriverAdapter, useValue: adapter)
      ]);
      var sampler = injector.get(Sampler);
      return sampler.sample();
    });
  }
}

var _DEFAULT_PROVIDERS = [
  Options.DEFAULT_PROVIDERS,
  Sampler.BINDINGS,
  ConsoleReporter.BINDINGS,
  RegressionSlopeValidator.BINDINGS,
  SizeValidator.BINDINGS,
  ChromeDriverExtension.BINDINGS,
  FirefoxDriverExtension.BINDINGS,
  IOsDriverExtension.BINDINGS,
  PerflogMetric.BINDINGS,
  SampleDescription.BINDINGS,
  MultiReporter.createBindings([ConsoleReporter]),
  MultiMetric.createBindings([PerflogMetric]),
  Reporter.bindTo(MultiReporter),
  Validator.bindTo(RegressionSlopeValidator),
  WebDriverExtension.bindTo(
      [ChromeDriverExtension, FirefoxDriverExtension, IOsDriverExtension]),
  Metric.bindTo(MultiMetric)
];
